
/**************************************************************************
**  (c) Copyright 2007, Andromeda Technology & Automation
** This is free software; you can redistribute it and/or modify it under the
** terms of the GNU General Public License, see the file COPYING.
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : df_filter.cpp
**      SYSTEM NAME    : 
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Transform output of 'df' into a Gnucomo XML document
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Oct 30, 2007
**      LAST UPDATE     : Nov 02, 2007
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: df_filter.cpp,v $
   Revision 1.1  2007/11/03 10:26:13  arjen
   Added a new filter which can directly read the output
   of the UNIX df command. A brief description is added in
   the user manual.

*****************************/

/* static const char *RCSID = "$Id: df_filter.cpp,v 1.1 2007/11/03 10:26:13 arjen Exp $"; */

#include <ctype.h>
#include <vector>

#include "df_filter.h"


std::vector<String> tokenize(String line)
{
   std::vector<String> words;
   int    l;           //  letter index in line

   l = 0;
   while (l < ~line)
   {
      int start, length;

      while (l < ~line && isspace(line[l]))
      {
         l++;
      }
      start = l;
      length = 0;

      while (l < ~line && !isspace(line[l]))
      {
         l++;
         length++;
      }
      String w = line(start, length);
      words.push_back(w);
   }
   return words;
}

//  Find a regular expression in a vector of Strings

int  findword(const std::vector<String> &tokens, const regex &pattern)
{
   int   i;

   i = 0;
   while (i < tokens.size() && !(tokens[i] == pattern))
   {
      i++;
   }

   if (i == tokens.size())
   {
      std::cerr << "Token not found.\n";
   }

   return i;
}

/*=========================================================================
**  NAME           : constructXML
**  SYNOPSIS       : int constructXML(message_buffer &in, strstream &xml)
**  PARAMETERS     : 
**  RETURN VALUE   : Create an XML document from output of 'df -k'
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Nov 02, 2007
**=========================================================================
*/

static const regex re_df_header("^Filesystem.+[Uu]se.+[Aa]vail.+Mounted");
static const regex re_df_i_header("^Filesystem +Inodes.+IUse.+Mounted");
static const regex re_df_data(".+ [0-9]+ +[0-9]+ +[0-9]+");

void df_filter::construct_XML(message_buffer &in, std::strstream &xml)
{
   String line;
   enum   df_report_detection { NONE, BLOCKS, INODES };

   df_report_detection  report_type = NONE;

   int    field_position[4];  //  Mountpoint, blocks, used, available

   std::vector<String>  tokens;

   scan_email_header(in);
   construct_header(xml);

   xml << "  <gcmt:data>\n";
   xml << "    <gcmt:parameters class='filesystem'>\n";

   while (in >> line)
   { 
      String  mountpoint;
      String  device;
      String  blocks;
      String  inodes;
      String  used;
      String  available;

      switch (report_type)
      {
      case NONE:  //  Header.
         if (line == re_df_header)
         {
            report_type = BLOCKS;
            tokens = tokenize(line);
            //  Make a map to find the fields we're interested in
            field_position[0] = findword(tokens, regex("Mounted"));
            field_position[1] = findword(tokens, regex("kbytes|-blocks"));
            field_position[2] = findword(tokens, regex("[uU]se"));
            field_position[3] = findword(tokens, regex("[aA]vail"));
         }
         if (line == re_df_i_header)
         {
            report_type = INODES;
            tokens = tokenize(line);
            //  Make a map to find the fields we're interested in
            field_position[0] = findword(tokens, regex("Mounted"));
            field_position[1] = findword(tokens, regex("Inodes"));
            field_position[2] = findword(tokens, regex("IUsed"));
            field_position[3] = findword(tokens, regex("IFree"));
         }
         break;

      case BLOCKS: // df output lines.
         tokens = tokenize(line);
         if (tokens.size() > 4)
         {
            mountpoint = tokens[field_position[0]];
            blocks     = tokens[field_position[1]];
            used       = tokens[field_position[2]];
            available  = tokens[field_position[3]];
            device     = tokens[0];
            if (mountpoint[0] == '/')
            {

               //   Create the XML element.

               xml << "    <gcmt:parameter name='" << mountpoint << "'>\n";
               xml << "      <gcmt:property name='device'>" << device << "</gcmt:property>\n";
               xml << "      <gcmt:property name='size'>" << blocks << "</gcmt:property>\n";
               xml << "      <gcmt:property name='used'>"   << used   << "</gcmt:property>\n";
               xml << "      <gcmt:property name='available'>" << available << "</gcmt:property>\n";
               xml << "    </gcmt:parameter>\n";
            }
         }
         else
         {
            //cerr << "Mismatch on " << line << "\n";
         }
         break;

      case INODES: // df output lines.
         tokens = tokenize(line);
         if (tokens.size() > 4)
         {
            mountpoint = tokens[field_position[0]];
            inodes     = tokens[field_position[1]];
            used       = tokens[field_position[2]];
            available  = tokens[field_position[3]];
            device     = tokens[0];
            if (mountpoint[0] == '/')
            {

               //   Create the XML element.

               xml << "    <gcmt:parameter name='" << mountpoint << "'>\n";
               xml << "      <gcmt:property name='device'>" << device << "</gcmt:property>\n";
               xml << "      <gcmt:property name='Inodes'>" << inodes << "</gcmt:property>\n";
               xml << "      <gcmt:property name='Iused'>"   << used   << "</gcmt:property>\n";
               xml << "      <gcmt:property name='Ifree'>" << available << "</gcmt:property>\n";
               xml << "    </gcmt:parameter>\n";
            }
         }
         else
         {
            //cerr << "Mismatch on " << line << "\n";
         }
         break;
      }
   }
   xml << "    </gcmt:parameters>\n";
   xml << "  </gcmt:data>\n";
   xml << "</gcmt:message>\n";
}

bool df_cooker::check_pattern(String logline)
{
   if (df_detected)
   {
      df_detected = (logline == re_df_data);
   }
   else
   {
      df_detected = (logline == re_df_header || logline == re_df_i_header);
   }

   return df_detected;
}

bool df_cooker::cook_this(String logline, UTC arrival)
{
   return true;
}
